﻿using System;
using System.Drawing;
using System.IO;
using System.Web.UI;
using System.Web.UI.WebControls;
using System.Collections.Generic;

/* Version 1.0 - BlogEngine.Net Gallery UserControl
 * 
 * By Paul Tumelty - February 5th 2008
 * -----------------------------------
 * Please feel free to contact me at paul@tumelty.co.uk if you have any ideas or suggestions for improvement.
 * If you find this control useful, or use it on your site, I'd love to hear from you!
 * 
 * This control has been reasonably well tested, although it was thrown together fairly quickly so apologies in advance for any bugs which may be present.
 * 
 * IMPORTANT: In order for Thumbnail creation to work correctly you will need to have write access to the directory in which the image is stored
 * 
 * Installation Instructions:
 * 
 * 1) Copy Gallery.ascx and Gallery.ascx.cs into your BlogEngine.Net UserControls directory
 * 2) Thats it!
 * 
 * To insert a gallery onto a blog post, or a page:
 * 
 * 1) Upload the pictures into a directory on your webserver.
 * 2) Create a new blog entry/page
 * 3) Insert the following code at the point where you would like the gallery
 *      [usercontrol: /blog/user controls/gallery.ascx ImageUrl=/path/to/images;RepeatColumns=6]
 * 
 * The following parameters are valid:
 * 
 * ImageUrl = The absolute Url path to your images i.e. /files/images/gallery
 * RepeatColumns = The number of pictures in each row of the gallery (Default is 1)
 * Width = The width of each thumbnail (Default is 75px)
 * LimitCount = The limits how many images are displayed
 * ShowInfo = If true shows image info filename file size etc.
 * GenerateThumbnails = True/False, when false, no thumbnails are generated/used (Default is true)
 * 
 */

/// <summary>
/// Represents a BlogEngine.Net Gallery
/// </summary>
public partial class Gallery : UserControl
{

    #region Internal Class to store details of images in gallery
    /// <summary>
    /// Internal Class to Represent an Image in the Galler
    /// </summary>
    public class GalleryImage
    {

        protected string _ImageUrl;
        protected string _LightBox;
        protected string _ThumbnailUrl;
        protected Unit _ImageWidth;
        protected Unit _ImageHeight;
        private FileInfo fileInfo;
        private bool showInfo;

        public string Filename
        {
            get
            {
                if (!this.showInfo) return string.Empty;
                return fileInfo.Exists ? System.IO.Path.GetFileNameWithoutExtension(fileInfo.Name) : string.Empty;
            }
        }

        public string LightBox
        {
            get { return string.IsNullOrEmpty(this._LightBox) ? string.Empty : string.Compare(this._LightBox.Trim(), "true", true) == 0 ? "lightbox" : "lightbox[" + this._LightBox + "]"; }
            set { _LightBox = value; }
        }
        
        public string Size
        {
            get
            {
                if (!this.showInfo) return string.Empty;
                if (fileInfo.Exists)
                {
                    double len = fileInfo.Length;
                    string end = "bytes";
                    if (len > 1024)
                    {
                        len = len / 1024;
                        end = "Kb";
                    }
                    if (len > 1024)
                    {
                        len = len / 1024;
                        end = "Mb";
                    }
                    len = Math.Round(len, 2);
                    return "Size: " + len.ToString() + end;
                }
                else
                {
                    return "Size: -1";
                }
            }
        }

        public string Dimentions
        {
            get
            {
                return string.Empty;
                if (this.showInfo)
                {
                    return "W: " + this._ImageWidth.ToString() + "<br/>H: " + this._ImageHeight.ToString();
                }
                else
                {
                    return string.Empty;
                }
            }
        }

        public string ImageUrl
        {
            get
            {
                return _ImageUrl;
            }
        }

        public string ThumbnailUrl
        {
            get
            {
                return _ThumbnailUrl;
            }
        }

        public Unit ImageWidth
        {
            get
            {
                return _ImageWidth;
            }
        }

        public Unit ImageHeight
        {
            get
            {
                return _ImageHeight;
            }
        }

        /// <summary>
        /// Initializes a new instance of the GalleryImage class.
        /// </summary>
        /// <param name="imageUrl">The URL to the Image</param>
        /// <param name="thumbnailUrl">The URL to the Thumbnail Image</param>
        /// <param name="imageWidth">The Width of the Image</param>
        /// <param name="imageHeight">The Height of the Image</param>
        /// 
        public GalleryImage(string imageUrl, string thumbnailUrl, int imageWidth, int imageHeight, string Filename, bool si)
        {
            _ImageUrl = imageUrl;
            _ThumbnailUrl = thumbnailUrl;

            _ImageWidth = new Unit(imageWidth);
            _ImageHeight = new Unit(imageHeight);
            fileInfo = new FileInfo(Filename);
            this.showInfo = si;
        }
    }
    #endregion

    #region Protected Variables
    /// <summary>
    /// The Url to the Images
    /// </summary>
    protected string _ImageUrl;

    /// <summary>
    /// The rel attribute for image links
    /// </summary>
    protected string _LightBox;

    /// <summary>
    /// The number of Columns
    /// </summary>
    protected int _RepeatColumns = 1;

    /// <summary>
    /// The Width of each thumbnail (default is 75px)
    /// </summary>
    protected int _Width = 75;

    /// <summary>
    /// The Height of each thumbnail
    /// </summary>
    protected int _Height;

    /// <summary>
    /// The limits how many images are displayed
    /// </summary>
    protected int _LimitCount = int.MaxValue;

    /// <summary>
    /// Should we try to create thumbnails?
    /// </summary>
    protected bool _GenerateThumbnails = true;

    /// <summary>
    /// Should we try to display image info?
    /// </summary>
    protected bool _ShowInfo = true;

    /// <summary>
    /// List of Images to include in the galleryImage
    /// </summary>
    protected List<GalleryImage> _Images = new List<GalleryImage>();

    /// <summary>
    /// Dictionary of valid Image Extensions
    /// </summary>
    protected static Dictionary<string, string> ImageExtensions = new Dictionary<string, string>();
    #endregion

    #region Public Properties
    /// <summary>
    /// The Height of the Image in the Gallery
    /// </summary>
    public int Height
    {
        get
        {
            return _Height;
        }
        set
        {
            _Height = value;
        }
    }

    /// <summary>
    /// The limits how many images are displayed
    /// </summary>
    public int LimitCount
    {
        get
        {
            return _LimitCount;
        }
        set
        {
            _LimitCount = value;
        }
    }

    /// <summary>
    /// The Width of the Image in the Gallery
    /// </summary>
    public int Width
    {
        get
        {
            return _Width;
        }
        set
        {
            _Width = value;
        }
    }

    /// <summary>
    /// The Url Path to the images containing the thumbnails.
    /// <example>/files/images/gallery</example>
    /// <remarks>A trailing / will automatically be added if this is omitted</remarks>
    /// </summary>
    public string ImageUrl
    {
        get
        {
            return _ImageUrl;
        }
        set
        {
            _ImageUrl = value.EndsWith("/") ? value : value + "/";
        }
    }

    public string LightBox
    {
        get { return _LightBox; }
        set { _LightBox = value; }
    }

    /// <summary>
    /// The number of columns to display in the gallery
    /// </summary>
    public int RepeatColumns
    {
        get
        {
            return _RepeatColumns;
        }
        set
        {
            _RepeatColumns = value;
        }
    }

    /// <summary>
    /// Should we try to generate Thumbnails?
    /// </summary>
    public bool GenerateThumbnails
    {
        get
        {
            return _GenerateThumbnails;
        }
        set
        {
            _GenerateThumbnails = value;
        }
    }


    /// <summary>
    /// Should we try to generate Thumbnails?
    /// </summary>
    public bool ShowInfo
    {
        get
        {
            return _ShowInfo;
        }
        set
        {
            _ShowInfo = value;
        }
    }
    #endregion

    #region Protected Helper Properties

    /// <summary>
    /// The Physical Path to the Images
    /// </summary>
    protected string ImagePath
    {
        get
        {
            return Server.MapPath(_ImageUrl);
        }
    }

    /// <summary>
    /// Url Path to the Thumbnail Images
    /// </summary>
    protected string ThumbnailUrl
    {
        get
        {
            return @"\thumbs\";// ImageUrl + "thumbs/";
        }
    }

    /// <summary>
    /// The Physical Path to the Thumbnail Images
    /// </summary>
    protected string ThumbnailPath
    {
        get
        {
            return Server.MapPath(ThumbnailUrl);
        }
    }
    #endregion

    #region Constructors
    /// <summary>
    /// Initialise Static Variables
    /// </summary>
    static Gallery()
    {

        // Add valid Image formats to extension list.
        ImageExtensions.Add(".jpg", "JPEG Image");
        ImageExtensions.Add(".gif", "GIF Image");
        ImageExtensions.Add(".png", "PNG Image");
    }
    #endregion

    #region Events
    protected void Page_Load(object sender, EventArgs e)
    {

        // Check to see if the Image Folder exists and if not, exit.
        if (!Directory.Exists(ImagePath))
        {
            Response.Write("Image folder does not exist " + this.ImagePath);
            return;
        }

        // Check to see if the Thumbnail Image Folder exists and if not, attempt to create it
        if (!File.Exists(ThumbnailPath))
        {
            try
            {
                Directory.CreateDirectory(ThumbnailPath);
            }
            catch
            {
                Response.Write("Cannot create Thumbnail Directory, check permissions");
            }
        }

        // Get directory objects for Image Folder and Thumbnail Folder
        DirectoryInfo oImageFolderInfo = new DirectoryInfo(this.ImagePath);
        DirectoryInfo oThumbnailFolderInfo = new DirectoryInfo(this.ThumbnailPath);

        FileInfo[] oFileList = oImageFolderInfo.GetFiles("*.*");
        int countItems = 0;

        string thumbsize = System.Configuration.ConfigurationManager.AppSettings["ThumbnailSize"];
        int thumbnailSize = 128;
        int.TryParse(thumbsize, out thumbnailSize);

        // Iterate around all the files in the image directory and look for images
        foreach (FileInfo oFile in oFileList)
        {

            string sImage = oFile.FullName;
            string sThumbnail = oFile.DirectoryName + @"\thumbs\" + oFile.Name;

            // If the image matches one of the valid extensions, add it into the list of images and create a thumbnail
            if (Gallery.ImageExtensions.ContainsKey(oFile.Extension.ToLower()))
            {
                GalleryImage galimg;
                galimg = null;
                // Are we generating thumbnails?
                if (this.GenerateThumbnails)
                {
                    ICSharpCode.SharpZipLib.Checksums.IChecksum crc;
                    crc = new ICSharpCode.SharpZipLib.Checksums.Crc32();
                    crc.Update(System.Text.Encoding.ASCII.GetBytes(sThumbnail));
                    long crcValue = crc.Value;
                    crc = null;
                    sThumbnail = Server.MapPath(@"\thumbs\") + crcValue.ToString() + System.IO.Path.GetExtension(sThumbnail);
                    sThumbnail = System.IO.Path.ChangeExtension(sThumbnail, ".png");

                    //Response.Write(sThumbnail);
                    //return;
                    FileInfo oThumbnailInfo = new FileInfo(sThumbnail);

                    // Build thumbnail if it doesn't exist
                    if (oThumbnailInfo.Exists == false)
                    {
                        Bitmap oImage = new Bitmap(sImage);

                        // Calculate Aspect Ratio
                        double aspectRatio = (double)oImage.Height / (double)oImage.Width;

                        Bitmap oThumbnailImage = ResizeBitmap(oImage, thumbnailSize, Convert.ToInt32(thumbnailSize * aspectRatio));

                        oThumbnailImage.Save(sThumbnail, System.Drawing.Imaging.ImageFormat.Png);
                    }

                    sThumbnail = this.ThumbnailUrl + System.IO.Path.GetFileName(sThumbnail);
                    //  Response.Write(sThumbnail);
                    //  return;
                    sThumbnail = sThumbnail.Replace(@"\", @"/");
                    // Add the Image to the Gallery with a thumbnail reference
                    galimg = new GalleryImage(this.ImageUrl + oFile.Name, sThumbnail.Replace(@"\", @"/"), this.Width, this.Height, oFile.FullName, this._ShowInfo);
                    galimg.LightBox = this._LightBox;
                    _Images.Add(galimg);

                }
                else
                {
                    // Add the Image to the Gallery with no thumbnail
                    galimg = new GalleryImage(this.ImageUrl + oFile.Name, this.ImageUrl + oFile.Name, this.Width, this.Height, oFile.FullName, this._ShowInfo);
                    galimg.LightBox = this._LightBox;
                    _Images.Add(galimg);

                }
                
            }

            countItems++;
            if (countItems >= this._LimitCount) break;
        }

        dlImages.RepeatColumns = _RepeatColumns;

        dlImages.DataSource = _Images;
        dlImages.DataBind();
    }
    #endregion

    #region Helper Functions
    /// <summary>
    /// Resizes a Bitmap to a given height/width
    /// </summary>
    /// <param name="oImage">Bitmap to Resize</param>
    /// <param name="nWidth">The Width</param>
    /// <param name="nHeight">The Height</param>
    /// <returns>A Thumbnail of the Image</returns>
    protected Bitmap ResizeBitmap(Bitmap oImage, int nWidth, int nHeight)
    {
        Bitmap oThumbnail = new Bitmap(nWidth, nHeight);

        // Get a Graphics Object from the Image
        using (Graphics oGraphics = Graphics.FromImage((System.Drawing.Image)oThumbnail))

            // Write a copy of the Image at the appropriate size
            oGraphics.DrawImage(oImage, 0, 0, nWidth, nHeight);

        // Return the Thumbnail
        return oThumbnail;
    }
    #endregion

}
// IChecksum.cs - Interface to compute a data checksum
// Copyright (C) 2001 Mike Krueger
//
// This file was translated from java, it was part of the GNU Classpath
// Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Linking this library statically or dynamically with other modules is
// making a combined work based on this library.  Thus, the terms and
// conditions of the GNU General Public License cover the whole
// combination.
// 
// As a special exception, the copyright holders of this library give you
// permission to link this library with independent modules to produce an
// executable, regardless of the license terms of these independent
// modules, and to copy and distribute the resulting executable under
// terms of your choice, provided that you also meet, for each linked
// independent module, the terms and conditions of the license of that
// module.  An independent module is a module which is not derived from
// or based on this library.  If you modify this library, you may extend
// this exception to your version of the library, but you are not
// obligated to do so.  If you do not wish to do so, delete this
// exception statement from your version.

namespace ICSharpCode.SharpZipLib.Checksums
{

    /// <summary>
    /// Interface to compute a data checksum used by checked input/output streams.
    /// A data checksum can be updated by one byte or with a byte array. After each
    /// update the value of the current checksum can be returned by calling
    /// <code>getValue</code>. The complete checksum object can also be reset
    /// so it can be used again with new data.
    /// </summary>
    public interface IChecksum
    {
        /// <summary>
        /// Returns the data checksum computed so far.
        /// </summary>
        long Value
        {
            get;
        }

        /// <summary>
        /// Resets the data checksum as if no update was ever called.
        /// </summary>
        void Reset();

        /// <summary>
        /// Adds one byte to the data checksum.
        /// </summary>
        /// <param name = "bval">
        /// the data value to add. The high byte of the int is ignored.
        /// </param>
        void Update(int bval);

        /// <summary>
        /// Updates the data checksum with the bytes taken from the array.
        /// </summary>
        /// <param name="buffer">
        /// buffer an array of bytes
        /// </param>
        void Update(byte[] buffer);

        /// <summary>
        /// Adds the byte array to the data checksum.
        /// </summary>
        /// <param name = "buf">
        /// the buffer which contains the data
        /// </param>
        /// <param name = "off">
        /// the offset in the buffer where the data starts
        /// </param>
        /// <param name = "len">
        /// the length of the data
        /// </param>
        void Update(byte[] buf, int off, int len);
    }
}
// CRC32.cs - Computes CRC32 data checksum of a data stream
// Copyright (C) 2001 Mike Krueger
//
// This file was translated from java, it was part of the GNU Classpath
// Copyright (C) 1999, 2000, 2001 Free Software Foundation, Inc.
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
//
// Linking this library statically or dynamically with other modules is
// making a combined work based on this library.  Thus, the terms and
// conditions of the GNU General Public License cover the whole
// combination.
// 
// As a special exception, the copyright holders of this library give you
// permission to link this library with independent modules to produce an
// executable, regardless of the license terms of these independent
// modules, and to copy and distribute the resulting executable under
// terms of your choice, provided that you also meet, for each linked
// independent module, the terms and conditions of the license of that
// module.  An independent module is a module which is not derived from
// or based on this library.  If you modify this library, you may extend
// this exception to your version of the library, but you are not
// obligated to do so.  If you do not wish to do so, delete this
// exception statement from your version.

//using System;

namespace ICSharpCode.SharpZipLib.Checksums
{

    /// <summary>
    /// Generate a table for a byte-wise 32-bit CRC calculation on the polynomial:
    /// x^32+x^26+x^23+x^22+x^16+x^12+x^11+x^10+x^8+x^7+x^5+x^4+x^2+x+1.
    ///
    /// Polynomials over GF(2) are represented in binary, one bit per coefficient,
    /// with the lowest powers in the most significant bit.  Then adding polynomials
    /// is just exclusive-or, and multiplying a polynomial by x is a right shift by
    /// one.  If we call the above polynomial p, and represent a byte as the
    /// polynomial q, also with the lowest power in the most significant bit (so the
    /// byte 0xb1 is the polynomial x^7+x^3+x+1), then the CRC is (q*x^32) mod p,
    /// where a mod b means the remainder after dividing a by b.
    ///
    /// This calculation is done using the shift-register method of multiplying and
    /// taking the remainder.  The register is initialized to zero, and for each
    /// incoming bit, x^32 is added mod p to the register if the bit is a one (where
    /// x^32 mod p is p+x^32 = x^26+...+1), and the register is multiplied mod p by
    /// x (which is shifting right by one and adding x^32 mod p if the bit shifted
    /// out is a one).  We start with the highest power (least significant bit) of
    /// q and repeat for all eight bits of q.
    ///
    /// The table is simply the CRC of all possible eight bit values.  This is all
    /// the information needed to generate CRC's on data a byte at a time for all
    /// combinations of CRC register values and incoming bytes.
    /// </summary>
    public sealed class Crc32 : IChecksum
    {
        readonly static uint CrcSeed = 0xFFFFFFFF;

        readonly static uint[] CrcTable = new uint[] {
			0x00000000, 0x77073096, 0xEE0E612C, 0x990951BA, 0x076DC419,
			0x706AF48F, 0xE963A535, 0x9E6495A3, 0x0EDB8832, 0x79DCB8A4,
			0xE0D5E91E, 0x97D2D988, 0x09B64C2B, 0x7EB17CBD, 0xE7B82D07,
			0x90BF1D91, 0x1DB71064, 0x6AB020F2, 0xF3B97148, 0x84BE41DE,
			0x1ADAD47D, 0x6DDDE4EB, 0xF4D4B551, 0x83D385C7, 0x136C9856,
			0x646BA8C0, 0xFD62F97A, 0x8A65C9EC, 0x14015C4F, 0x63066CD9,
			0xFA0F3D63, 0x8D080DF5, 0x3B6E20C8, 0x4C69105E, 0xD56041E4,
			0xA2677172, 0x3C03E4D1, 0x4B04D447, 0xD20D85FD, 0xA50AB56B,
			0x35B5A8FA, 0x42B2986C, 0xDBBBC9D6, 0xACBCF940, 0x32D86CE3,
			0x45DF5C75, 0xDCD60DCF, 0xABD13D59, 0x26D930AC, 0x51DE003A,
			0xC8D75180, 0xBFD06116, 0x21B4F4B5, 0x56B3C423, 0xCFBA9599,
			0xB8BDA50F, 0x2802B89E, 0x5F058808, 0xC60CD9B2, 0xB10BE924,
			0x2F6F7C87, 0x58684C11, 0xC1611DAB, 0xB6662D3D, 0x76DC4190,
			0x01DB7106, 0x98D220BC, 0xEFD5102A, 0x71B18589, 0x06B6B51F,
			0x9FBFE4A5, 0xE8B8D433, 0x7807C9A2, 0x0F00F934, 0x9609A88E,
			0xE10E9818, 0x7F6A0DBB, 0x086D3D2D, 0x91646C97, 0xE6635C01,
			0x6B6B51F4, 0x1C6C6162, 0x856530D8, 0xF262004E, 0x6C0695ED,
			0x1B01A57B, 0x8208F4C1, 0xF50FC457, 0x65B0D9C6, 0x12B7E950,
			0x8BBEB8EA, 0xFCB9887C, 0x62DD1DDF, 0x15DA2D49, 0x8CD37CF3,
			0xFBD44C65, 0x4DB26158, 0x3AB551CE, 0xA3BC0074, 0xD4BB30E2,
			0x4ADFA541, 0x3DD895D7, 0xA4D1C46D, 0xD3D6F4FB, 0x4369E96A,
			0x346ED9FC, 0xAD678846, 0xDA60B8D0, 0x44042D73, 0x33031DE5,
			0xAA0A4C5F, 0xDD0D7CC9, 0x5005713C, 0x270241AA, 0xBE0B1010,
			0xC90C2086, 0x5768B525, 0x206F85B3, 0xB966D409, 0xCE61E49F,
			0x5EDEF90E, 0x29D9C998, 0xB0D09822, 0xC7D7A8B4, 0x59B33D17,
			0x2EB40D81, 0xB7BD5C3B, 0xC0BA6CAD, 0xEDB88320, 0x9ABFB3B6,
			0x03B6E20C, 0x74B1D29A, 0xEAD54739, 0x9DD277AF, 0x04DB2615,
			0x73DC1683, 0xE3630B12, 0x94643B84, 0x0D6D6A3E, 0x7A6A5AA8,
			0xE40ECF0B, 0x9309FF9D, 0x0A00AE27, 0x7D079EB1, 0xF00F9344, 
			0x8708A3D2, 0x1E01F268, 0x6906C2FE, 0xF762575D, 0x806567CB,
			0x196C3671, 0x6E6B06E7, 0xFED41B76, 0x89D32BE0, 0x10DA7A5A,
			0x67DD4ACC, 0xF9B9DF6F, 0x8EBEEFF9, 0x17B7BE43, 0x60B08ED5,
			0xD6D6A3E8, 0xA1D1937E, 0x38D8C2C4, 0x4FDFF252, 0xD1BB67F1,
			0xA6BC5767, 0x3FB506DD, 0x48B2364B, 0xD80D2BDA, 0xAF0A1B4C,
			0x36034AF6, 0x41047A60, 0xDF60EFC3, 0xA867DF55, 0x316E8EEF,
			0x4669BE79, 0xCB61B38C, 0xBC66831A, 0x256FD2A0, 0x5268E236,
			0xCC0C7795, 0xBB0B4703, 0x220216B9, 0x5505262F, 0xC5BA3BBE,
			0xB2BD0B28, 0x2BB45A92, 0x5CB36A04, 0xC2D7FFA7, 0xB5D0CF31,
			0x2CD99E8B, 0x5BDEAE1D, 0x9B64C2B0, 0xEC63F226, 0x756AA39C,
			0x026D930A, 0x9C0906A9, 0xEB0E363F, 0x72076785, 0x05005713,
			0x95BF4A82, 0xE2B87A14, 0x7BB12BAE, 0x0CB61B38, 0x92D28E9B,
			0xE5D5BE0D, 0x7CDCEFB7, 0x0BDBDF21, 0x86D3D2D4, 0xF1D4E242,
			0x68DDB3F8, 0x1FDA836E, 0x81BE16CD, 0xF6B9265B, 0x6FB077E1,
			0x18B74777, 0x88085AE6, 0xFF0F6A70, 0x66063BCA, 0x11010B5C,
			0x8F659EFF, 0xF862AE69, 0x616BFFD3, 0x166CCF45, 0xA00AE278,
			0xD70DD2EE, 0x4E048354, 0x3903B3C2, 0xA7672661, 0xD06016F7,
			0x4969474D, 0x3E6E77DB, 0xAED16A4A, 0xD9D65ADC, 0x40DF0B66,
			0x37D83BF0, 0xA9BCAE53, 0xDEBB9EC5, 0x47B2CF7F, 0x30B5FFE9,
			0xBDBDF21C, 0xCABAC28A, 0x53B39330, 0x24B4A3A6, 0xBAD03605,
			0xCDD70693, 0x54DE5729, 0x23D967BF, 0xB3667A2E, 0xC4614AB8,
			0x5D681B02, 0x2A6F2B94, 0xB40BBE37, 0xC30C8EA1, 0x5A05DF1B, 
			0x2D02EF8D
		};

        /// <summary>
        /// The crc data checksum so far.
        /// </summary>
        uint crc = 0;

        /// <summary>
        /// Returns the CRC32 data checksum computed so far.
        /// </summary>
        public long Value
        {
            get
            {
                return (long)crc;
            }
        }

        /// <summary>
        /// Resets the CRC32 data checksum as if no update was ever called.
        /// </summary>
        public void Reset()
        {
            crc = 0;
        }

        /// <summary>
        /// Updates the checksum with the int bval.
        /// </summary>
        /// <param name = "bval">
        /// the byte is taken as the lower 8 bits of bval
        /// </param>
        public void Update(int bval)
        {
            crc ^= CrcSeed;
            crc = CrcTable[(crc ^ bval) & 0xFF] ^ (crc >> 8);
            crc ^= CrcSeed;
        }

        /// <summary>
        /// Updates the checksum with the bytes taken from the array.
        /// </summary>
        /// <param name="buffer">
        /// buffer an array of bytes
        /// </param>
        public void Update(byte[] buffer)
        {
            Update(buffer, 0, buffer.Length);
        }

        /// <summary>
        /// Adds the byte array to the data checksum.
        /// </summary>
        /// <param name = "buf">
        /// the buffer which contains the data
        /// </param>
        /// <param name = "off">
        /// the offset in the buffer where the data starts
        /// </param>
        /// <param name = "len">
        /// the length of the data
        /// </param>
        public void Update(byte[] buf, int off, int len)
        {
            if (buf == null)
            {
                throw new ArgumentNullException("buf");
            }

            if (off < 0 || len < 0 || off + len > buf.Length)
            {
                throw new ArgumentOutOfRangeException();
            }

            crc ^= CrcSeed;

            while (--len >= 0)
            {
                crc = CrcTable[(crc ^ buf[off++]) & 0xFF] ^ (crc >> 8);
            }

            crc ^= CrcSeed;
        }
    }
}
